#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "../include/common.h"
#include "../include/libPIF.h"

int main(int argc, char **argv) {
  forPifHead_t  thisHead;
  int mode, ncol, nrow, nsec, ncenx, nceny, ncenz, start=1;
  int i, j, k, m, ir, indx, n, peak_loc, outer_loc, inner_loc, *nann;
  int first_flip, nimg;
  float *map3d, r2, bkg;
  double avg, std, numerator, denominator, peak_val;
  double sum, sum2;
  double *sumann, *sum2ann, *avgann, *stdann, *nstdann;

  if(argc != 2) {
    printf("\nUSAGE:  imgstats map/stack name\n");
    printf("prints out statistics on a PIF map or image stack\n\n");
    exit(0);
  }
  
  thisHead.luNum = 4;
  pif_open(argv[1], &thisHead.luNum, "r");
  pif_read_gh(&thisHead);

  nimg = thisHead.nimg;
  mode = thisHead.mode;
  ncol = thisHead.numCols;
  nrow = thisHead.numRows;
  nsec = thisHead.numSecs;
  ncenx = ncol/2; // center for count starting from 0
  nceny = nrow/2; // center for count starting from 0
  ncenz = nsec/2; // center for count starting from 0

  if(mode != 20 && mode != 40 && mode != 46) {
    printf("Invalid map mode %d - mode must equal 20, 40, or 46\n");
    exit(0);
  }

  map3d    = (float *) malloc(sizeof(float)*ncol*nrow*nsec);
  sumann   = (double *) malloc(sizeof(double)*(ncenx+1));
  sum2ann  = (double *) malloc(sizeof(double)*(ncenx+1));
  avgann   = (double *) malloc(sizeof(double)*(ncenx+1));
  stdann   = (double *) malloc(sizeof(double)*(ncenx+1));
  nstdann  = (double *) malloc(sizeof(double)*(ncenx+1));
  nann     = (int *) malloc(sizeof(int)*(ncenx+1));

  for(m=1; m<=nimg; m++) {
    
    thisHead.imgNum = m;
    pif_read_dh(&thisHead);
    thisHead.luNum = 4;
    bkg = thisHead.img_bkg;
    
    if(mode == 20)
      pif_read_mapi2(&thisHead, map3d, &start, &nsec, &ncol, &nrow, &nsec);
    else if(mode == 40)
      pif_read_mapi4(&thisHead, map3d, &start, &nsec, &ncol, &nrow, &nsec);
    else if(mode == 46) {
      pif_read_boxi4(&thisHead, map3d);
      for(i=0; i<ncol*nrow; i++)
	map3d[i] -= bkg;
    }

    // Initialize the accumulators before processing each image
    sum = 0.0;
    sum2 = 0.0;
    for(ir=0; ir<=ncenx; ir++) {
      sumann[ir]  = 0.0;
      sum2ann[ir] = 0.0;
      avgann[ir]  = 0.0;
      stdann[ir]  = 0.0;
      nstdann[ir] = 0.0;
      nann[ir]    = 0;
    }

    for(i=0; i<ncol; i++) {
      for(j=0; j<nrow; j++) {
	for(k=0; k<nsec; k++) {
	  indx = i*nrow*nsec + j*nsec + k;
	  r2 = (i-ncenx)*(i-ncenx)+(j-nceny)*(j-nceny)+(k-ncenz)*(k-ncenz);
	  ir = sqrt(r2);
	  sum += map3d[indx];
	  sum2 += map3d[indx]*map3d[indx];
	  if(ir <= ncenx) {
	    nann[ir]++;
	    sumann[ir] += map3d[indx];
	    sum2ann[ir] += map3d[indx]*map3d[indx];
	  }
	}
      }
    }
    
    // Global statistics
    n = ncol*nrow*nsec;
    avg = sum/n;
    numerator = sum2 - (sum*sum)/n;
    denominator = n-1;
    std = sqrt(numerator/denominator);
    
    peak_loc = 0;
    peak_val = 1.0e10;
    
    // Statistics by annulus
    for(ir=0; ir<=ncenx; ir++) {
      if(nann[ir] > 0)
	avgann[ir] = sumann[ir]/nann[ir];
      else
	avgann[ir] = 0.0;
      
      if(nann[ir] > 1) {
	numerator = sum2ann[ir] - (sumann[ir]*sumann[ir])/nann[ir];
	denominator = nann[ir]-1;
	stdann[ir] = sqrt(numerator/denominator);
      } else
	stdann[ir] = 0.0;
      
      if(avgann[ir] < peak_val) {
	peak_val = avgann[ir];
	peak_loc = ir;
      }
      
      if(stdann[ir] > 0.0)
	nstdann[ir] = fabs(stdann[ir]/avgann[ir]);
      else
	nstdann[ir] = 0.0;
    }
    
    // Identify local maxima on either side of peak
    outer_loc = peak_loc;
    for (ir=peak_loc+1; ir<ncenx; ir++) {
      if(avgann[ir] < avgann[ir-1])
	break;
      outer_loc = ir;
    }
    
    inner_loc = peak_loc;
    for (ir=peak_loc-1; ir>0; ir--) {
      if(avgann[ir] < avgann[ir+1])
	break;
      inner_loc = ir;
    }
    
    first_flip = peak_loc;
    for (ir=peak_loc+1; ir<ncenx; ir++) {
      if(avgann[ir] > 0)
	break;
      first_flip = ir;
    }
    
    printf("--- Global map statistics ---\n");
    printf("mode = %d\n", mode);
    printf("numCols = %d\n", ncol);
    printf("numRows = %d\n", nrow);
    printf("numSecs = %d\n", nsec);
    printf("avg = %10.5f\n", avg);
    printf("std = %10.5f\n", std);
    printf("radius at peak density = %d\n", peak_loc);
    printf("avg at peak density = %10.5f\n", avgann[peak_loc]);
    printf("std at peak density = %10.5f\n", stdann[peak_loc]);
    printf("first local peak  < peak_loc = %d\n", inner_loc);
    printf("first local_peak  > peak_loc = %d\n", outer_loc);
    printf("first sign change > peak_loc = %d\n", first_flip);
    printf("\n--- Statistics by spherical annulus ---\n");
    printf(" r(pix)   count        avg        std    |std/ann|\n");
    for(ir=0; ir<=ncenx; ir++)
      printf("%5d   %8d  %10.5f %10.5f %10.5f\n", ir, nann[ir], avgann[ir], 
	     stdann[ir], nstdann[ir]);
    
  }

  pif_close(&thisHead.luNum);
}
